@@ -1,8 +1,8 @@
-package Compute::Max;
+package MonitisMonitorManager::Compute::Max;
 use strict;
-use Carp;
-use Data::Dumper;
+require List::Util;
 use List::Util qw(max);
+use Carp;
 my %max_hash = ();
 
 sub new {
