@@ -1,9 +1,7 @@
-package Execution::SNMP;
+package MonitisMonitorManager::Execution::SNMP;
 use strict;
 use MonitisMonitorManager::M3PluginCommon;
-use Carp;
-use Data::Dumper;
-use Net::SNMP;
+require Net::SNMP;
 
 sub new {
 	my ($class, $name) = @_;
@@ -45,14 +43,14 @@ sub execute {
 	);
 
 	if (!defined $session) {
-		carp "SNMP Error: ", $error;
+		MonitisMonitorManager::M3PluginCommon::log_message("err", "SNMP Error: " . $error);
 		return "";
 	}
 
 	my $result = $session->get_request(-varbindlist => [ $oid ],);
 
 	if (!defined $result) {
-		carp "SNMP Error: ", $session->error();
+		MonitisMonitorManager::M3PluginCommon::log_message("err", "SNMP Error: " . $session->error());
 		$session->close();
 		return "";
 	}
