@@ -1,7 +1,7 @@
 package MonitisMonitorManager::M3PluginCommon;
 use strict;
-no strict "refs";
 use Carp;
+use Sys::Syslog;
 
 # returns the parameter specified or croaks on an error
 sub get_mandatory_parameter {
@@ -33,4 +33,9 @@ sub get_optional_parameter {
 	}
 }
 
+sub log_message {
+	my ($priority, $message) = @_;
+	MonitisMonitorManager::M3Logger->instance()->log_message($priority, $message);
+}
+
 1;
